package parser_test

import (
	"testing"

	"git.sharkk.net/Sharkk/Mako/parser"
)

func TestVariableTypeHints(t *testing.T) {
	tests := []struct {
		input    string
		variable string
		typeHint parser.Type
		hasHint  bool
		desc     string
	}{
		{"x = 42", "x", parser.TypeUnknown, false, "no type hint"},
		{"x: number = 42", "x", parser.TypeNumber, true, "number type hint"},
		{"name: string = \"hello\"", "name", parser.TypeString, true, "string type hint"},
		{"flag: bool = true", "flag", parser.TypeBool, true, "bool type hint"},
		{"data: table = {}", "data", parser.TypeTable, true, "table type hint"},
		{"fn_var: function = fn() end", "fn_var", parser.TypeFunction, true, "function type hint"},
		{"value: any = nil", "value", parser.TypeAny, true, "any type hint"},
		{"ptr: nil = nil", "ptr", parser.TypeNil, true, "nil type hint"},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			l := parser.NewLexer(tt.input)
			p := parser.NewParser(l)
			program := p.ParseProgram()
			checkParserErrors(t, p)

			if len(program.Statements) != 1 {
				t.Fatalf("expected 1 statement, got %d", len(program.Statements))
			}

			stmt, ok := program.Statements[0].(*parser.Assignment)
			if !ok {
				t.Fatalf("expected Assignment, got %T", program.Statements[0])
			}

			// Check variable name
			ident, ok := stmt.Target.(*parser.Identifier)
			if !ok {
				t.Fatalf("expected Identifier for Target, got %T", stmt.Target)
			}

			if ident.Value != tt.variable {
				t.Errorf("expected variable %s, got %s", tt.variable, ident.Value)
			}

			// Check type hint
			if tt.hasHint {
				if stmt.TypeHint.Type == parser.TypeUnknown {
					t.Error("expected type hint but got TypeUnknown")
				} else {
					if stmt.TypeHint.Type != tt.typeHint {
						t.Errorf("expected type hint %v, got %v", tt.typeHint, stmt.TypeHint.Type)
					}
					if stmt.TypeHint.Inferred {
						t.Error("expected type hint to not be inferred")
					}
				}
			} else {
				if stmt.TypeHint.Type != parser.TypeUnknown {
					t.Errorf("expected no type hint but got %v", stmt.TypeHint.Type)
				}
			}
		})
	}
}

func TestFunctionParameterTypeHints(t *testing.T) {
	tests := []struct {
		input  string
		params []struct {
			name     string
			typeHint parser.Type
		}
		returnType parser.Type
		hasReturn  bool
		desc       string
	}{
		{
			"fn(a, b) end",
			[]struct {
				name     string
				typeHint parser.Type
			}{
				{"a", parser.TypeUnknown},
				{"b", parser.TypeUnknown},
			},
			parser.TypeUnknown, false,
			"no type hints",
		},
		{
			"fn(a: number, b: string) end",
			[]struct {
				name     string
				typeHint parser.Type
			}{
				{"a", parser.TypeNumber},
				{"b", parser.TypeString},
			},
			parser.TypeUnknown, false,
			"parameter type hints only",
		},
		{
			"fn(x: number): string end",
			[]struct {
				name     string
				typeHint parser.Type
			}{
				{"x", parser.TypeNumber},
			},
			parser.TypeString, true,
			"parameter and return type hints",
		},
		{
			"fn(): bool end",
			[]struct {
				name     string
				typeHint parser.Type
			}{},
			parser.TypeBool, true,
			"return type hint only",
		},
		{
			"fn(a: number, b, c: string): table end",
			[]struct {
				name     string
				typeHint parser.Type
			}{
				{"a", parser.TypeNumber},
				{"b", parser.TypeUnknown},
				{"c", parser.TypeString},
			},
			parser.TypeTable, true,
			"mixed parameter types with return",
		},
		{
			"fn(callback: function, data: any): nil end",
			[]struct {
				name     string
				typeHint parser.Type
			}{
				{"callback", parser.TypeFunction},
				{"data", parser.TypeAny},
			},
			parser.TypeNil, true,
			"function and any types",
		},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			l := parser.NewLexer(tt.input)
			p := parser.NewParser(l)
			expr := p.ParseExpression(parser.LOWEST)
			checkParserErrors(t, p)

			fn, ok := expr.(*parser.FunctionLiteral)
			if !ok {
				t.Fatalf("expected FunctionLiteral, got %T", expr)
			}

			// Check parameters
			if len(fn.Parameters) != len(tt.params) {
				t.Fatalf("expected %d parameters, got %d", len(tt.params), len(fn.Parameters))
			}

			for i, expected := range tt.params {
				param := fn.Parameters[i]
				if param.Name != expected.name {
					t.Errorf("parameter %d: expected name %s, got %s", i, expected.name, param.Name)
				}

				if expected.typeHint == parser.TypeUnknown {
					if param.TypeHint.Type != parser.TypeUnknown {
						t.Errorf("parameter %d: expected no type hint but got %v", i, param.TypeHint.Type)
					}
				} else {
					if param.TypeHint.Type == parser.TypeUnknown {
						t.Errorf("parameter %d: expected type hint %v but got TypeUnknown", i, expected.typeHint)
					} else if param.TypeHint.Type != expected.typeHint {
						t.Errorf("parameter %d: expected type hint %v, got %v", i, expected.typeHint, param.TypeHint.Type)
					}
				}
			}

			// Check return type
			if tt.hasReturn {
				if fn.ReturnType.Type == parser.TypeUnknown {
					t.Error("expected return type hint but got TypeUnknown")
				} else if fn.ReturnType.Type != tt.returnType {
					t.Errorf("expected return type %v, got %v", tt.returnType, fn.ReturnType.Type)
				}
			} else {
				if fn.ReturnType.Type != parser.TypeUnknown {
					t.Errorf("expected no return type but got %v", fn.ReturnType.Type)
				}
			}
		})
	}
}

func TestTypeHintStringRepresentation(t *testing.T) {
	tests := []struct {
		input    string
		expected string
		desc     string
	}{
		{"x: number = 42", "local x: number = 42.00", "typed variable assignment"},
		{"fn(a: number, b: string): bool end", "fn(a: number, b: string): bool\nend", "typed function"},
		{"callback: function = fn(x: number): string return \"\" end", "local callback: function = fn(x: number): string\n\treturn \"\"\nend", "typed function assignment"},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			l := parser.NewLexer(tt.input)
			p := parser.NewParser(l)

			var result string
			if tt.input[0] == 'f' { // function literal
				expr := p.ParseExpression(parser.LOWEST)
				checkParserErrors(t, p)
				result = expr.String()
			} else { // assignment statement
				program := p.ParseProgram()
				checkParserErrors(t, p)
				result = program.Statements[0].String()
			}

			if result != tt.expected {
				t.Errorf("expected %q, got %q", tt.expected, result)
			}
		})
	}
}

func TestTypeHintValidation(t *testing.T) {
	tests := []struct {
		input         string
		expectedError string
		desc          string
	}{
		{"x: invalid = 42", "invalid type name 'invalid'", "invalid type name"},
		{"x: = 42", "expected type name after ':'", "missing type name"},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			l := parser.NewLexer(tt.input)
			p := parser.NewParser(l)
			p.ParseProgram()

			if !p.HasErrors() {
				t.Fatal("expected parsing errors")
			}

			errors := p.Errors()
			found := false
			for _, err := range errors {
				if err.Message == tt.expectedError {
					found = true
					break
				}
			}

			if !found {
				errorMsgs := make([]string, len(errors))
				for i, err := range errors {
					errorMsgs[i] = err.Message
				}
				t.Errorf("expected error %q, got %v", tt.expectedError, errorMsgs)
			}
		})
	}
}

func TestMemberAccessWithoutTypeHints(t *testing.T) {
	tests := []struct {
		input string
		desc  string
	}{
		{"table.key = 42", "dot notation assignment"},
		{"arr[1] = \"hello\"", "bracket notation assignment"},
		{"obj.nested.deep = true", "chained dot assignment"},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			l := parser.NewLexer(tt.input)
			p := parser.NewParser(l)
			program := p.ParseProgram()
			checkParserErrors(t, p)

			if len(program.Statements) != 1 {
				t.Fatalf("expected 1 statement, got %d", len(program.Statements))
			}

			stmt, ok := program.Statements[0].(*parser.Assignment)
			if !ok {
				t.Fatalf("expected Assignment, got %T", program.Statements[0])
			}

			// Member access should never have type hints
			if stmt.TypeHint.Type != parser.TypeUnknown {
				t.Error("member access assignment should not have type hints")
			}

			// Should not be a declaration
			if stmt.IsDeclaration {
				t.Error("member access assignment should not be a declaration")
			}
		})
	}
}

func TestTypeInferenceEngine(t *testing.T) {
	input := `x: number = 42
name: string = "hello"
fn_var: function = fn(a: number, b: string): bool
	result: bool = a > 0
	return result
end
echo name`

	l := parser.NewLexer(input)
	p := parser.NewParser(l)
	program := p.ParseProgram()
	checkParserErrors(t, p)

	// Run type inference
	inferrer := parser.NewTypeInferrer()
	typeErrors := inferrer.InferTypes(program)

	// Should have no type errors for valid code
	if len(typeErrors) > 0 {
		errorMsgs := make([]string, len(typeErrors))
		for i, err := range typeErrors {
			errorMsgs[i] = err.Error()
		}
		t.Errorf("unexpected type errors: %v", errorMsgs)
	}
}

func TestTypeInferenceErrors(t *testing.T) {
	tests := []struct {
		input         string
		expectedError string
		desc          string
	}{
		{
			"x: number = \"hello\"",
			"type mismatch in assignment",
			"type mismatch in assignment",
		},
		{
			"x = 42\ny: string = x",
			"type mismatch in assignment",
			"type mismatch with inferred type",
		},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			l := parser.NewLexer(tt.input)
			p := parser.NewParser(l)
			program := p.ParseProgram()
			checkParserErrors(t, p)

			inferrer := parser.NewTypeInferrer()
			typeErrors := inferrer.InferTypes(program)

			if len(typeErrors) == 0 {
				t.Fatal("expected type errors")
			}

			found := false
			for _, err := range typeErrors {
				if containsSubstring(err.Message, tt.expectedError) {
					found = true
					break
				}
			}

			if !found {
				errorMsgs := make([]string, len(typeErrors))
				for i, err := range typeErrors {
					errorMsgs[i] = err.Message
				}
				t.Errorf("expected error containing %q, got %v", tt.expectedError, errorMsgs)
			}
		})
	}
}

func TestVariadicFunctionTypeHints(t *testing.T) {
	tests := []struct {
		input    string
		variadic bool
		desc     string
	}{
		{"fn(a: number, ...) end", true, "variadic after typed param"},
		{"fn(...) end", true, "variadic only"},
		{"fn(a: number, b: string) end", false, "no variadic"},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			l := parser.NewLexer(tt.input)
			p := parser.NewParser(l)
			expr := p.ParseExpression(parser.LOWEST)
			checkParserErrors(t, p)

			fn, ok := expr.(*parser.FunctionLiteral)
			if !ok {
				t.Fatalf("expected FunctionLiteral, got %T", expr)
			}

			if fn.Variadic != tt.variadic {
				t.Errorf("expected variadic = %t, got %t", tt.variadic, fn.Variadic)
			}
		})
	}
}

func TestComplexTypeHintProgram(t *testing.T) {
	input := `config: table = {
	host = "localhost",
	port = 8080,
	enabled = true
}

handler: function = fn(request: table, callback: function): nil
	status: number = 200
	if request.method == "GET" then
		response: table = {status = status, body = "OK"}
		result = callback(response)
	end
end

server: table = {
	config = config,
	handler = handler,
	start = fn(self: table): bool
		return true
	end
}`

	l := parser.NewLexer(input)
	p := parser.NewParser(l)
	program := p.ParseProgram()
	checkParserErrors(t, p)

	if len(program.Statements) != 3 {
		t.Fatalf("expected 3 statements, got %d", len(program.Statements))
	}

	// Check first statement: config table with typed assignments
	configStmt, ok := program.Statements[0].(*parser.Assignment)
	if !ok {
		t.Fatalf("expected Assignment, got %T", program.Statements[0])
	}

	if configStmt.TypeHint.Type == parser.TypeUnknown || configStmt.TypeHint.Type != parser.TypeTable {
		t.Error("expected table type hint for config")
	}

	// Check second statement: handler function with typed parameters
	handlerStmt, ok := program.Statements[1].(*parser.Assignment)
	if !ok {
		t.Fatalf("expected Assignment, got %T", program.Statements[1])
	}

	if handlerStmt.TypeHint.Type == parser.TypeUnknown || handlerStmt.TypeHint.Type != parser.TypeFunction {
		t.Error("expected function type hint for handler")
	}

	fn, ok := handlerStmt.Value.(*parser.FunctionLiteral)
	if !ok {
		t.Fatalf("expected FunctionLiteral value, got %T", handlerStmt.Value)
	}

	if len(fn.Parameters) != 2 {
		t.Fatalf("expected 2 parameters, got %d", len(fn.Parameters))
	}

	// Check parameter types
	if fn.Parameters[0].TypeHint.Type == parser.TypeUnknown || fn.Parameters[0].TypeHint.Type != parser.TypeTable {
		t.Error("expected table type for request parameter")
	}

	if fn.Parameters[1].TypeHint.Type == parser.TypeUnknown || fn.Parameters[1].TypeHint.Type != parser.TypeFunction {
		t.Error("expected function type for callback parameter")
	}

	// Check return type
	if fn.ReturnType.Type == parser.TypeUnknown || fn.ReturnType.Type != parser.TypeNil {
		t.Error("expected nil return type for handler")
	}

	// Check third statement: server table
	serverStmt, ok := program.Statements[2].(*parser.Assignment)
	if !ok {
		t.Fatalf("expected Assignment, got %T", program.Statements[2])
	}

	if serverStmt.TypeHint.Type == parser.TypeUnknown || serverStmt.TypeHint.Type != parser.TypeTable {
		t.Error("expected table type hint for server")
	}
}

func TestTypeInfoInterface(t *testing.T) {
	// Test that all expression types properly implement TypeInfo()
	expressions := []parser.Expression{
		&parser.Identifier{Value: "x"},
		&parser.NumberLiteral{Value: 42},
		&parser.StringLiteral{Value: "hello"},
		&parser.BooleanLiteral{Value: true},
		&parser.NilLiteral{},
		&parser.TableLiteral{},
		&parser.FunctionLiteral{},
		&parser.CallExpression{Function: &parser.Identifier{Value: "fn"}},
		&parser.PrefixExpression{Operator: "-", Right: &parser.NumberLiteral{Value: 1}},
		&parser.InfixExpression{Left: &parser.NumberLiteral{Value: 1}, Operator: "+", Right: &parser.NumberLiteral{Value: 2}},
		&parser.IndexExpression{Left: &parser.Identifier{Value: "arr"}, Index: &parser.NumberLiteral{Value: 0}},
		&parser.DotExpression{Left: &parser.Identifier{Value: "obj"}, Key: "prop"},
	}

	for i, expr := range expressions {
		t.Run(string(rune('0'+i)), func(t *testing.T) {
			// Should have default type initially
			typeInfo := expr.TypeInfo()

			// Basic literals should have their expected types
			switch e := expr.(type) {
			case *parser.NumberLiteral:
				if typeInfo.Type != parser.TypeNumber {
					t.Errorf("expected number type, got %v", typeInfo.Type)
				}
			case *parser.StringLiteral:
				if typeInfo.Type != parser.TypeString {
					t.Errorf("expected string type, got %v", typeInfo.Type)
				}
			case *parser.BooleanLiteral:
				if typeInfo.Type != parser.TypeBool {
					t.Errorf("expected bool type, got %v", typeInfo.Type)
				}
			case *parser.NilLiteral:
				if typeInfo.Type != parser.TypeNil {
					t.Errorf("expected nil type, got %v", typeInfo.Type)
				}
			case *parser.TableLiteral:
				if typeInfo.Type != parser.TypeTable {
					t.Errorf("expected table type, got %v", typeInfo.Type)
				}
			case *parser.FunctionLiteral:
				if typeInfo.Type != parser.TypeFunction {
					t.Errorf("expected function type, got %v", typeInfo.Type)
				}
			case *parser.Identifier:
				// Identifiers default to any type
				if typeInfo.Type != parser.TypeAny {
					t.Errorf("expected any type for untyped identifier, got %v", typeInfo.Type)
				}
			default:
				// Other expressions may have unknown type initially
				_ = e
			}
		})
	}
}
